#!/usr/bin/env python
# -*- coding: utf_8 -*-
#
# @author  killdream <killdreamxd@gmail.com>
# @version 0.2.1 alpha
#
# Copyright (C) 2008 Quildreen <killdreamxd@gmail.com>
# -----                                                   
# This program is free software; you can redistribute  
# it and/or modify it under the terms of the GNU       
# General Public License as published by the Free      
# Software Foundation; either version 2 of the License,
# or (at your option) any later version.               
#                                                         
# This program is distributed in the hope that it will 
# be useful, but WITHOUT ANY WARRANTY; without even    
# the implied warranty of MERCHANTABILITY or FITNESS   
# FOR A PARTICULAR PURPOSE. See the GNU General Public 
# License for more details.                            
#                                                         
# You should have received a copy of the GNU General   
# Public License along with this program; if not,      
# write to the Free Software Foundation, Inc., 675     
# Mass Ave, Cambridge, MA 02139, USA.

# Import pygame modules
import pygame
from pygame.locals import *

# Import some utility modules
import math
import core
import random
random.seed()

class oFrame:
    def __init__(self, image, **kwargs):
        self.image = image
        self.mask = kwargs.pop("mask", None)
        self.hotspot = kwargs.pop("hotspot", (0,0))
        self.actionpoint = kwargs.pop("actionpoint", self.hotspot)
        

class oSprite:
    def __init__(self):
        self.x = 0
        self.y = 0
        self.speed = .15
        self.speedMul = 1
        self.direction = "N"
        self.dead = False
        self.godmode = 0
        self.updated = False
        
        # sort of State Machine Animation
        self.animState = None
        self.animFrame = 0
        self.animLooped = True
        self.animSpeed = 0
        self.animFrames = []
        self.animFinished = False
        self.animations = {}
        
        # dirty rect
        self.last_rect = None
        
        # ai function
        self.AI = None
        self.OnAnimationEnd = None
    
    def rx(self):
        return self.x - self.hotspot()[0]
    
    def ry(self):
        return self.y - self.hotspot()[1]
    
    def hotspot(self):
        return self.getFrame().hotspot
    
    def action_point(self):
        return self.getFrame().actionpoint
    
    def getFrame(self, frame=None):
        if frame is None:
            frame = self.animFrame
            
        frame = int( math.floor(frame) )
        if frame < len(self.animFrames) and frame >= 0:
            return self.animFrames[frame]
        return oFrame(None)
        
    def getFrameImage(self, frame=None):
        return self.getFrame(frame).image
        
    def getMask(self, frame=None):
        return self.getFrame(frame).mask
        
    def canWalk(self):
        canWalk = core.level.walkable(self.rx(), self.ry(), self.getMask())
        frame = self.getFrameImage()
        if frame is None:
            w, h = 0
        else:
            w, h = frame.get_size()
        if self.rx() < 0 or self.ry() < 0 or self.rx() + w > 800 or self.ry() + h > 600:
            canWalk = False
        
        return canWalk
        
    
    def animFromStrip(self, image, mappings, width, height, mask=None):
        gw = int(image.get_width() / width)
        gh = int(image.get_height() / height)
        for mapping in mappings:
            hotspot = (0,0)
            actionpoint = (0,0)
            
            if len(mapping) == 6:
                anim, direction, start, count, speed, looped = mapping
            elif len(mapping) == 7:
                anim, direction, start, count, speed, looped, hotspot = mapping
            elif len(mapping) == 8:
                anim, direction, start, count, speed, looped, hotspot, actionpoint = mapping
                
            frames = [ ]
            for index in range(count):
                x = int(( start + index ) % gw) * width
                y = int(( start + index ) / gw) * height
                rect = pygame.Rect(int(x), int(y), width, height)
                frame = oFrame( core.im_crop(image, rect), hotspot=hotspot, actionpoint=actionpoint )
                if mask is not None:
                    frame.mask = pygame.mask.from_surface(core.im_crop(mask, rect), 0)
                frames.append(frame)
            
            if not self.animations.has_key(anim):
                self.animations[anim] = { }
                
            self.animations[anim].update({direction: [speed, looped, frames]})
                
                
    def draw(self, surface):        
        frame = self.getFrameImage()
        if frame is not None:
            surface.blit(frame, (int(self.rx()), int(self.ry())))
    
    def update(self, lag):
        if self.godmode > 0:
            self.godmode -= lag
        if self.godmode < 0:
            self.godmode = 0
        
        if callable(self.AI):
            self.AI(self, lag)
            
        last_frame = math.floor(self.animFrame)
        self.animFrame += self.animSpeed * lag
        if self.animFrame >= len(self.animFrames):
            if self.animLooped:
                self.animFrame = 0
            else:
                self.animFrame = len(self.animFrames)-1
                self.animFinished = True
                if callable(self.OnAnimationEnd):
                    self.OnAnimationEnd()
            
        # return the dirty rects
        last_rect    = self.last_rect
        current_rect = None
        
        frame = self.getFrameImage()
        if frame is not None:
            current_rect = pygame.Rect((self.rx(), self.ry()), frame.get_size()) 
            
        self.last_rect = current_rect
        return [last_rect, current_rect]
        
    def die(self):
        self.dead = True
        
    def changeState(self, newState):
        if newState == self.animState:
            return
        
        if self.animations.has_key(newState):
            self.animFinished = False
            self.animState = newState
            self.animFrame = 0
            anim_info  = self.animations[self.animState][self.direction]
            self.animSpeed, self.animLooped, self.animFrames = anim_info
            
    def changeDirection(self, newDirection):
        if newDirection == self.direction:
            return
        
        if newDirection in ["N","E","S","W"]:
            if self.animations.has_key(self.animState) and\
             self.animations[self.animState].has_key(self.direction):
                self.animFinished = False
                self.animFrame = 0
                self.direction = newDirection
                anim_info = self.animations[self.animState][self.direction]
                self.animSpeed, self.animLooped, self.animFrames = anim_info
            
            
class oSpriteGroup:
    def __init__(self):
        self.objects = []
        
    def yComp(self, s1, s2):
        if s1.y < s2.y:
            return -1
        elif s1.y == s2.y:
            return 0
        else:
            return 1
        
    def clear(self):
        for sprite in self.objects:
            sprite.die()
        self.objects = []
        
    def add(self, *sprites):
        for sprite in sprites:
            if not sprite in self.objects:
                self.objects.append(sprite)
        self.objects.sort( self.yComp )
        
    def remove(self, *sprites):
        for sprite in sprites:
            if sprite in self.objects:
                self.objects.remove(sprite)
        self.objects.sort( self.yComp )
        
    def checkCollision(self, kill1=False, kill2=False, sprite=None, *groups):
        collisions = []
        # Shouldn't do this, but...
        if sprite is not None:
            sprites = [sprite]
        else:
            sprites = self.objects
            
        for sprite in sprites:
            for group in groups:
                for _sprite in group.objects:
                    if sprite is not _sprite and not (sprite.dead or _sprite.dead) and not\
                       (sprite.animState == "dying" or _sprite.animState == "dying") and\
                       sprite.godmode == 0 and _sprite.godmode == 0:
                        if sprite.getFrameImage() is not None and _sprite.getFrameImage() is not None:
                            r1 = sprite.getFrameImage().get_rect(topleft=(sprite.rx(),sprite.ry()))
                            r2 = _sprite.getFrameImage().get_rect(topleft=(_sprite.rx(),_sprite.ry()))
                            if r1.colliderect(r2):
                                collisioned = True
                                if sprite.getMask() is not None and _sprite.getMask() is not None:
                                    collisioned = sprite.getMask().overlap(_sprite.getMask(),
                                     (int(sprite.rx()-_sprite.rx()), int(sprite.ry()-_sprite.ry()))) is not None
                                    
                                if collisioned:    
                                    collisions.append([_sprite, sprite])
                                    if kill1:
                                        sprite.dead = True
                                    if kill2:
                                        _sprite.dead = True
        return collisions
                            
    
    def draw(self, surface):
        for sprite in self.objects:
            sprite.updated = False
            if not sprite.dead and sprite.draw:
                sprite.draw(surface)
            
    def update(self, lag):
        dirty_rect = []
        dead_sprites = []
        needsResort = False
                
        for sprite in self.objects:
            if sprite.dead:
                dead_sprites.append(sprite)
                continue
            
            if not sprite.updated:
                sprite.updated = True
                y = sprite.y
                dirty_rect.extend(sprite.update(lag))
                _y = sprite.y
                if _y != y:
                    needsResort = True
            
        self.remove(*dead_sprites)
            
        if needsResort:
            self.objects.sort( self.yComp )
        
        return dirty_rect
            
            
def bugAI(bug, lag):
    if bug.animState == "dying":
        return
    
    ly = bug.y
    lx = bug.x
    if bug.direction == "N":
        bug.y -= bug.speed*bug.speedMul * lag
    if bug.direction == "S":
        bug.y += bug.speed*bug.speedMul * lag
    if bug.direction == "E":
        bug.x += bug.speed*bug.speedMul * lag
    if bug.direction == "W":
        bug.x -= bug.speed*bug.speedMul * lag
    
    if not bug.canWalk():
        bug.x = lx
        bug.y = ly
        change_dir_prob = 100
    else:
        change_dir_prob = 1
        
    if random.randint(0,100) < change_dir_prob:
        bug.changeDirection(random.choice(["N","W","S","E"]))
    